/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import net.minecraft.block.Block;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.GuiCreateWorld;
import net.minecraft.client.resources.I18n;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.WorldType;
import net.minecraft.world.chunk.Chunk;
import net.minecraftforge.client.event.GuiScreenEvent;
import net.minecraftforge.client.event.RenderGameOverlayEvent;
import net.minecraftforge.event.entity.player.ItemTooltipEvent;
import net.minecraftforge.fml.client.registry.RenderingRegistry;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.oredict.OreDictionary;

import net.dries007.tfc.ConfigTFC;
import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.capability.forge.CapabilityForgeable;
import net.dries007.tfc.api.capability.forge.IForgeable;
import net.dries007.tfc.api.capability.heat.CapabilityItemHeat;
import net.dries007.tfc.api.capability.heat.IItemHeat;
import net.dries007.tfc.api.capability.size.CapabilityItemSize;
import net.dries007.tfc.api.capability.size.IItemSize;
import net.dries007.tfc.api.util.IMetalObject;
import net.dries007.tfc.api.util.IRockObject;
import net.dries007.tfc.client.render.RenderFallingBlockTFC;
import net.dries007.tfc.objects.entity.EntityFallingBlockTFC;
import net.dries007.tfc.world.classic.CalenderTFC;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataProvider;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;
import static net.minecraft.util.text.TextFormatting.*;

@Mod.EventBusSubscriber(value = Side.CLIENT, modid = MOD_ID)
public class ClientEvents
{
    public static void preInit()
    {
        RenderingRegistry.registerEntityRenderingHandler(EntityFallingBlockTFC.class, RenderFallingBlockTFC::new);
    }

    @SideOnly(Side.CLIENT)
    @SubscribeEvent(priority = EventPriority.HIGH)
    public static void openGUI(GuiScreenEvent.InitGuiEvent.Pre event)
    {
        if (ConfigTFC.CLIENT.makeWorldTypeClassicDefault && event.getGui() instanceof GuiCreateWorld)
        {
            GuiCreateWorld gui = ((GuiCreateWorld) event.getGui());
            // Only change if default is selected, because coming back from customisation, this will be set already.
            if (gui.field_146331_K == WorldType.field_77137_b.func_82747_f())
                gui.field_146331_K = TerraFirmaCraft.getWorldTypeTFC().func_82747_f();
        }
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void onRenderGameOverlayText(RenderGameOverlayEvent.Text event)
    {
        Minecraft mc = Minecraft.func_71410_x();
        List<String> list = event.getRight();
        if (ConfigTFC.GENERAL.debug && mc.field_71474_y.field_74330_P)
        {
            //noinspection ConstantConditions
            BlockPos blockpos = new BlockPos(mc.func_175606_aa().field_70165_t, mc.func_175606_aa().func_174813_aQ().field_72338_b, mc.func_175606_aa().field_70161_v);
            Chunk chunk = mc.field_71441_e.func_175726_f(blockpos);
            if (mc.field_71441_e.func_175667_e(blockpos) && !chunk.func_76621_g())
            {
                final int x = blockpos.func_177958_n() & 15, z = blockpos.func_177952_p() & 15;
                ChunkDataTFC data = chunk.getCapability(ChunkDataProvider.CHUNK_DATA_CAPABILITY, null);

                list.add("");
                list.add(AQUA + "TerraFirmaCraft");

                if (data == null || !data.isInitialized()) list.add("No data ?!");
                else
                {
                    list.add(String.format("%sTemps: Base: %s%.1f°%s Biome Avg: %s%.1f°%s Actual: %s%.1f°",
                        GRAY, WHITE, data.getBaseTemp(), GRAY,
                        WHITE, data.getAverageTemp(), GRAY,
                        WHITE, ClimateTFC.getHeightAdjustedTemp(mc.field_71441_e, blockpos)
                    ));
                    list.add(String.format("%sTime: %s%02d:%02d %04d/%02d/%02d",
                        GRAY, WHITE,
                        CalenderTFC.getHourOfDay(),
                        CalenderTFC.getMinuteOfHour(),
                        CalenderTFC.getTotalYears(),
                        CalenderTFC.getMonthOfYear().id(),
                        CalenderTFC.getDayOfMonth()
                        )
                    );

                    list.add(GRAY + "Biome: " + WHITE + mc.field_71441_e.func_180494_b(blockpos).func_185359_l());

                    list.add(GRAY + "Rainfall: " + WHITE + data.getRainfall());
                    list.add(GRAY + "Flora Density: " + WHITE + data.getFloraDensity());
                    list.add(GRAY + "Flora Diversity: " + WHITE + data.getFloraDiversity());

                    list.add(GRAY + "Valid Trees: ");
                    data.getValidTrees().forEach(t -> list.add(String.format("%s %s (%.1f)", WHITE, t.getRegistryName(), t.getDominance())));

                    //list.add(GRAY + "Rocks: " + WHITE + data.getRockLayer1(x, z).name + ", " + data.getRockLayer2(x, z).name + ", " + data.getRockLayer3(x, z).name);
                    //list.add(GRAY + "Stability: " + WHITE + data.getStabilityLayer(x, z).name);
                    //list.add(GRAY + "Drainage: " + WHITE + data.getDrainageLayer(x, z).name);
                    list.add(GRAY + "Sea level offset: " + WHITE + data.getSeaLevelOffset(x, z));
                    //list.add(GRAY + "Fish population: " + WHITE + data.getFishPopulation());

                    //list.add("");
                    //list.add(GRAY + "Rock at feet: " + WHITE + data.getRockLayerHeight(x, blockpos.getY(), z).name);

                    // list.add("");
                    //data.getOresSpawned().stream().map(String::valueOf).forEach(list::add);
                }
            }
        }
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public static void onItemTooltip(ItemTooltipEvent event)
    {
        ItemStack stack = event.getItemStack();
        if (stack.func_190926_b())
        {
            TerraFirmaCraft.getLog().warn("ItemTooltipEvent with empty stack??", new Exception());
            return;
        }
        Item item = stack.func_77973_b();
        List<String> tt = event.getToolTip();

        // Stuff that should always be shown as part of the tooltip
        IItemSize size = CapabilityItemSize.getIItemSize(stack);
        if (size != null)
        {
            size.addSizeInfo(stack, tt);
        }
        IItemHeat heat = stack.getCapability(CapabilityItemHeat.ITEM_HEAT_CAPABILITY, null);
        if (heat != null)
        {
            heat.addHeatInfo(stack, tt);
        }

        if (event.getFlags().func_194127_a()) // Only added with advanced tooltip mode
        {
            if (item instanceof IMetalObject)
            {
                ((IMetalObject) item).addMetalInfo(stack, tt);
            }
            else if (item instanceof ItemBlock)
            {
                Block block = ((ItemBlock) item).func_179223_d();
                if (block instanceof IMetalObject)
                {
                    ((IMetalObject) block).addMetalInfo(stack, tt);
                }
            }
            if (item instanceof IRockObject)
            {
                ((IRockObject) item).addRockInfo(stack, tt);
            }
            else if (item instanceof ItemBlock)
            {
                Block block = ((ItemBlock) item).func_179223_d();
                if (block instanceof IRockObject)
                {
                    ((IRockObject) block).addRockInfo(stack, tt);
                }
            }

            // todo: remove this debug tooltip
            if (stack.hasCapability(CapabilityForgeable.FORGEABLE_CAPABILITY, null))
            {
                IForgeable cap = stack.getCapability(CapabilityForgeable.FORGEABLE_CAPABILITY, null);
                assert cap != null;
                tt.add("Forge Stuff: " + cap.serializeNBT());
            }

            if (ConfigTFC.CLIENT.showToolClassTooltip)
            {
                Set<String> toolClasses = item.getToolClasses(stack);
                if (toolClasses.size() == 1)
                {
                    tt.add(I18n.func_135052_a("tfc.tooltip.toolclass", toolClasses.iterator().next()));
                }
                else if (toolClasses.size() > 1)
                {
                    tt.add(I18n.func_135052_a("tfc.tooltip.toolclasses"));
                    for (String toolClass : toolClasses)
                    {
                        tt.add("+ " + toolClass);
                    }
                }
            }
            if (ConfigTFC.CLIENT.showOreDictionaryTooltip)
            {
                int[] ids = OreDictionary.getOreIDs(stack);
                if (ids.length == 1)
                {
                    tt.add(I18n.func_135052_a("tfc.tooltip.oredictionaryentry", OreDictionary.getOreName(ids[0])));
                }
                else if (ids.length > 1)
                {
                    tt.add(I18n.func_135052_a("tfc.tooltip.oredictionaryentries"));
                    ArrayList<String> names = new ArrayList<>(ids.length);
                    for (int id : ids)
                    {
                        names.add("+ " + OreDictionary.getOreName(id));
                    }
                    names.sort(null); // Natural order (String.compare)
                    tt.addAll(names);
                }
            }
            if (ConfigTFC.CLIENT.showNBTTooltip)
            {
                if (stack.func_77942_o())
                {
                    tt.add("NBT: " + stack.func_77978_p());
                }
            }
        }
    }
}
