/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui;

import java.io.IOException;

import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.recipes.AnvilRecipe;
import net.dries007.tfc.client.button.GuiButtonAnvilPlanIcon;
import net.dries007.tfc.client.button.IButtonTooltip;
import net.dries007.tfc.network.PacketGuiButton;
import net.dries007.tfc.objects.te.TEAnvilTFC;
import net.dries007.tfc.util.NBTBuilder;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;
import static net.dries007.tfc.objects.te.TEAnvilTFC.SLOT_INPUT_1;

@SideOnly(Side.CLIENT)
public class GuiAnvilPlan extends GuiContainerTE<TEAnvilTFC>
{
    public static final ResourceLocation PLAN_BACKGROUND = new ResourceLocation(MOD_ID, "textures/gui/anvil_plan.png");

    private final ItemStack inputStack;

    public GuiAnvilPlan(Container container, InventoryPlayer playerInv, TEAnvilTFC tile)
    {
        super(container, playerInv, tile, PLAN_BACKGROUND);

        IItemHandler cap = tile.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
        this.inputStack = cap == null ? ItemStack.field_190927_a : cap.getStackInSlot(SLOT_INPUT_1);
    }

    @Override
    public void func_73866_w_()
    {
        super.func_73866_w_();

        int buttonID = -1;
        int posX = 7;
        int posY = 25;
        for (AnvilRecipe recipe : AnvilRecipe.getAllFor(inputStack))
        {
            func_189646_b(new GuiButtonAnvilPlanIcon(recipe, ++buttonID, field_147003_i + posX, field_147009_r + posY));
            posX += 18;
            if (posX == 169)
            {
                posY += 18;
                posX = 7;
            }
        }
    }

    @Override
    protected void func_191948_b(int mouseX, int mouseY)
    {
        // Button Tooltips
        for (GuiButton button : field_146292_n)
        {
            if (button instanceof IButtonTooltip && button.func_146115_a())
            {
                IButtonTooltip tooltip = (IButtonTooltip) button;
                if (tooltip.hasTooltip())
                {
                    func_146279_a(I18n.func_135052_a(tooltip.getTooltip()), mouseX, mouseY);
                }
            }
        }
        super.func_191948_b(mouseX, mouseY);
    }

    @Override
    protected void func_146976_a(float partialTicks, int mouseX, int mouseY)
    {
        GlStateManager.func_179131_c(1, 1, 1, 1);
        field_146297_k.func_110434_K().func_110577_a(background);
        func_73729_b(field_147003_i, field_147009_r, 0, 0, field_146999_f, field_147000_g);
    }

    @Override
    protected void func_146979_b(int mouseX, int mouseY)
    {
        String name = I18n.func_135052_a("tfc.tooltip.anvil_plan") + ": " + I18n.func_135052_a(inputStack.func_77977_a() + ".name");
        field_146289_q.func_78276_b(name, field_146999_f / 2 - field_146289_q.func_78256_a(name) / 2, 6, 0x404040);

        super.func_146979_b(mouseX, mouseY);
    }

    @Override
    protected void func_146284_a(GuiButton button) throws IOException
    {
        if (button instanceof GuiButtonAnvilPlanIcon)
        {
            // This fires when you select a plan in the Plan GUI
            TerraFirmaCraft.getLog().info("Pressed the plan button");
            ResourceLocation recipeName = ((GuiButtonAnvilPlanIcon) button).getRecipeName();
            TerraFirmaCraft.getNetwork().sendToServer(new PacketGuiButton(button.field_146127_k, new NBTBuilder().setString("recipe", recipeName.toString()).build()));
        }
        super.func_146284_a(button);
    }
}
