/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui;

import net.minecraft.client.gui.GuiButton;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.client.event.GuiScreenEvent;
import net.minecraftforge.common.MinecraftForge;

import net.dries007.tfc.api.recipes.KnappingRecipe;
import net.dries007.tfc.client.button.GuiButtonKnapping;
import net.dries007.tfc.objects.container.ContainerKnapping;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

public class GuiKnapping extends GuiContainerTFC
{
    private static final ResourceLocation BG_TEXTURE = new ResourceLocation(MOD_ID, "textures/gui/knapping.png");
    private static final ResourceLocation ALT_BG_TEXTURE = new ResourceLocation(MOD_ID, "textures/gui/knapping_clay.png");
    private final ResourceLocation buttonTexture;

    public GuiKnapping(Container container, EntityPlayer player, KnappingRecipe.Type type, ResourceLocation buttonTexture)
    {
        super(container, player.field_71071_by, type == KnappingRecipe.Type.CLAY || type == KnappingRecipe.Type.FIRE_CLAY ? ALT_BG_TEXTURE : BG_TEXTURE);
        this.buttonTexture = buttonTexture;
        field_147000_g = 184; // Bigger than normal gui
    }

    @Override
    public void func_73866_w_()
    {
        super.func_73866_w_();
        for (int x = 0; x < 5; x++)
        {
            for (int y = 0; y < 5; y++)
            {
                int bx = (field_146294_l - field_146999_f) / 2 + 12 + 16 * x;
                int by = (field_146295_m - field_147000_g) / 2 + 12 + 16 * y;
                func_189646_b(new GuiButtonKnapping(x + 5 * y, bx, by, 16, 16, buttonTexture));
            }
        }
    }

    @Override
    protected void func_146976_a(float partialTicks, int mouseX, int mouseY)
    {
        // Check if the container has been updated
        if (field_147002_h instanceof ContainerKnapping && ((ContainerKnapping) field_147002_h).requiresReset)
        {
            for (GuiButton button : field_146292_n)
            {
                if (button instanceof GuiButtonKnapping)
                {
                    button.field_146125_m = false;
                }
            }
            ((ContainerKnapping) field_147002_h).requiresReset = false;
        }
        super.func_146976_a(partialTicks, mouseX, mouseY);
    }

    @Override
    protected void func_146284_a(GuiButton button)
    {
        if (button instanceof GuiButtonKnapping)
        {
            ((GuiButtonKnapping) button).onClick();
            button.func_146113_a(field_146297_k.func_147118_V());
        }
    }

    @Override
    protected void func_146273_a(int mouseX, int mouseY, int clickedMouseButton, long timeSinceLastClick)
    {
        if (clickedMouseButton == 0)
        {
            for (GuiButton button : this.field_146292_n)
            {
                if (button instanceof GuiButtonKnapping && button.func_146116_c(field_146297_k, mouseX, mouseY))
                {
                    GuiScreenEvent.ActionPerformedEvent.Pre event = new GuiScreenEvent.ActionPerformedEvent.Pre(this, button, field_146292_n);
                    if (MinecraftForge.EVENT_BUS.post(event))
                        break;
                    else if (field_146290_a == event.getButton())
                        continue;

                    field_146290_a = event.getButton();
                    event.getButton().func_146116_c(field_146297_k, mouseX, mouseY);
                    func_146284_a(event.getButton());

                    MinecraftForge.EVENT_BUS.post(new GuiScreenEvent.ActionPerformedEvent.Post(this, event.getButton(), field_146292_n));
                }
            }
        }
    }
}
