/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.registries.TFCRegistries;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.objects.items.metal.ItemIngot;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class TEIngotPile extends TileEntity
{
    private Metal metal;
    private int count;

    public TEIngotPile()
    {
        metal = Metal.UNKNOWN;
        count = 1;
    }

    @Override
    public void func_145839_a(NBTTagCompound tag)
    {
        metal = TFCRegistries.METALS.getValue(new ResourceLocation(tag.func_74779_i("metal")));
        count = tag.func_74762_e("count");
        super.func_145839_a(tag);
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag)
    {
        tag.func_74778_a("metal", (metal == null) ? Metal.UNKNOWN.toString() : metal.getRegistryName().toString());
        tag.func_74768_a("count", count);
        return super.func_189515_b(tag);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public double func_145833_n()
    {
        return 1024.0D;
    }

    @Override
    @Nullable
    public SPacketUpdateTileEntity func_189518_D_()
    {
        if (field_145850_b != null)
        {
            return new SPacketUpdateTileEntity(this.func_174877_v(), 0, this.func_189515_b(new NBTTagCompound()));
        }

        return null;
    }

    @Override
    public NBTTagCompound func_189517_E_()
    {
        // The tag from this method is used for the initial chunk packet, and it needs to have the TE position!
        NBTTagCompound nbt = new NBTTagCompound();
        nbt.func_74768_a("x", this.func_174877_v().func_177958_n());
        nbt.func_74768_a("y", this.func_174877_v().func_177956_o());
        nbt.func_74768_a("z", this.func_174877_v().func_177952_p());
        return func_189515_b(nbt);

    }

    @Override
    public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity packet)
    {
        this.handleUpdateTag(packet.func_148857_g());
    }

    @Override
    public void handleUpdateTag(NBTTagCompound tag)
    {
        func_145839_a(tag);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox()
    {
        return new AxisAlignedBB(func_174877_v(), func_174877_v().func_177963_a(1D, 1D, 1D));
    }

    public void onBreakBlock()
    {
        InventoryHelper.func_180173_a(field_145850_b, field_174879_c.func_177958_n(), field_174879_c.func_177956_o(), field_174879_c.func_177952_p(),
            new ItemStack(ItemIngot.get(metal, Metal.ItemType.INGOT), count));
    }

    public Metal getMetal()
    {
        return metal;
    }

    public void setMetal(Metal metal)
    {
        this.metal = metal;
    }

    public int getCount()
    {
        return count;
    }

    public void setCount(int count)
    {
        this.count = count;
    }
}
