/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.ItemStackHandler;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.objects.inventory.capability.ItemStackHandlerTE;

/**
 * This is a helper class for TE's with a simple inventory that will respect automation
 * To provide side based automation, you must expose a IItemHandler wrapper based on the input side
 * Without overriding the getCapability methods, this will not accept items from external automation
 */
@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public abstract class TEInventory extends TileEntity
{
    protected final ItemStackHandler inventory;

    TEInventory(int inventorySize)
    {
        super();
        inventory = new ItemStackHandlerTE(this, inventorySize);
        this.func_70296_d();
    }

    public void setAndUpdateSlots(int slot)
    {
        this.func_70296_d();
    }

    public int getSlotLimit(int slot)
    {
        return 64;
    }

    public boolean isItemValid(int slot, ItemStack stack) { return true; }

    @Override
    public void func_145839_a(NBTTagCompound compound)
    {
        inventory.deserializeNBT(compound.func_74775_l("inventory"));
        super.func_145839_a(compound);
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound compound)
    {
        compound.func_74782_a("inventory", inventory.serializeNBT());
        return super.func_189515_b(compound);
    }

    @Override
    @Nullable
    public SPacketUpdateTileEntity func_189518_D_()
    {
        NBTTagCompound updateTagDescribingTileEntityState = func_189517_E_();
        return new SPacketUpdateTileEntity(this.field_174879_c, 1, updateTagDescribingTileEntityState);
    }

    @Override
    public NBTTagCompound func_189517_E_()
    {
        NBTTagCompound nbtTagCompound = new NBTTagCompound();
        func_189515_b(nbtTagCompound);
        return nbtTagCompound;
    }

    @Override
    public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity pkt)
    {
        NBTTagCompound updateTagDescribingTileEntityState = pkt.func_148857_g();
        handleUpdateTag(updateTagDescribingTileEntityState);
    }

    @Override
    public void handleUpdateTag(NBTTagCompound tag)
    {
        this.func_145839_a(tag);
    }

    @Override
    public boolean hasCapability(Capability<?> capability, @Nullable EnumFacing facing)
    {
        return (capability == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY && facing == null) || super.hasCapability(capability, facing);
    }

    @Override
    @SuppressWarnings("unchecked")
    public <T> T getCapability(Capability<T> capability, @Nullable EnumFacing facing)
    {
        if (capability == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY && facing == null)
        {
            return (T) inventory;
        }
        return super.getCapability(capability, facing);
    }

    public void onBreakBlock(World world, BlockPos pos)
    {
        for (int i = 0; i < inventory.getSlots(); i++)
        {
            InventoryHelper.func_180173_a(world, pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), inventory.getStackInSlot(i));
        }
    }
}
